package com.rshy.project.hy.server;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.rshy.project.hy.baseRe.BaseException;
import com.rshy.project.hy.model.constant.WebConstant;
import com.rshy.project.hy.server.dto.OrderDTO;
import com.rshy.project.hy.server.dto.RetrievalDTO;
import com.rshy.project.hy.server.param.RetrievaLoginParam;
import com.rshy.project.hy.server.param.RetrievalRepositoryParam;
import com.rshy.project.hy.server.vo.RepositoryVO;
import com.rshy.project.hy.server.vo.RetrievaResultVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: lileilei
 * @date: 2022/4/20 10:11
 * @description: 依图人脸检索第三方服务
 */
@Component
@Slf4j
public class YituFaceServer {
    /**
     * 授权凭证请求头
     */
    private static final String HEADERS_KEY = "session_id";
    /**
     * 人像库标识
     */
    private static final String REPOSITORY_KEY = "repository_ids";

    @Value("${face.retrieva.loginUrl}")
    private String loginUrl;
    @Value("${face.retrieva.repositoryUrl}")
    private String repositoryUrl;
    @Value("${face.retrieva.retrievalUrl}")
    private String retrievalUrl;
    @Value("${face.retrieva.userInfo.name}")
    private String name;
    @Value("${face.retrieva.userInfo.password}")
    private String password;
    @Value("${face.retrieva.userInfo.threshold}")
    private String threshold;

    /**
     * 创建缓存，默认15分钟过期
     *
     * @see <a href="https://www.hutool.cn/docs/#/cache/TimedCache">定时缓存</a>
     */
    private static TimedCache<String, String> timedCache = CacheUtil.newTimedCache(15 * 60 * 1000);


    /**
     * 登录接口
     *
     * @param retrievaLoginParam
     * @return
     */
    public RetrievaResultVO login(RetrievaLoginParam retrievaLoginParam) {
        String body = timedCache.get(HEADERS_KEY, false);

        if (StrUtil.isEmpty(body)) {
            body = HttpUtil.post(loginUrl, JSON.toJSONString(retrievaLoginParam));
            timedCache.put(HEADERS_KEY, body);
        }

        return JSON.parseObject(body, RetrievaResultVO.class);
    }

    /**
     * 登录接口
     *
     * @param retrievaLoginParam
     * @param timeout            豪秒
     * @return
     */
    public RetrievaResultVO login(RetrievaLoginParam retrievaLoginParam, Integer timeout) {
        String body = timedCache.get(HEADERS_KEY, false);

        if (StrUtil.isEmpty(body)) {
            body = HttpUtil.post(loginUrl, JSON.toJSONString(retrievaLoginParam), timeout);
            timedCache.put(HEADERS_KEY, body);
        }

        return JSON.parseObject(body, RetrievaResultVO.class);
    }

    /**
     * 登录接口
     *
     * @return
     */
    public RetrievaResultVO login() {
        return this.login(new RetrievaLoginParam().setName(name).setPassword(password));
    }

    /**
     * 登录接口
     *
     * @param timeout
     * @return
     */
    public RetrievaResultVO login(Integer timeout) {
        return this.login(new RetrievaLoginParam().setName(name).setPassword(password), timeout);
    }

    /**
     * 获取人像库
     *
     * @param sessionId 会话标识
     */
    public RetrievaResultVO repository(String sessionId) {
        return this.repository(sessionId, WebConstant.TIMEOUT * 1000);
    }

    /**
     * 获取人像库
     *
     * @param sessionId 会话标识
     */
    public RetrievaResultVO repository(String sessionId, Integer timeout) {
        String body = timedCache.get(REPOSITORY_KEY, false);
        if (StrUtil.isEmpty(body)) {
            body = HttpRequest.get(repositoryUrl).header(HEADERS_KEY, sessionId).timeout(timeout).execute().body();
            //缓存一天失效
            timedCache.put(REPOSITORY_KEY, body, 24 * 60 * 60 * 1000);
        }

        return JSON.parseObject(body, RetrievaResultVO.class);
    }


    /**
     * 获取人像库
     */
    public RetrievaResultVO repository() {
        return this.repository(WebConstant.TIMEOUT * 1000);
    }

    /**
     * 获取人像库
     *
     * @param timeout 秒
     */
    public RetrievaResultVO repository(Integer timeout) {
        RetrievaResultVO login = this.login(timeout);
        log.info("依图 login: {}", login.toString());
        if (login.getRtn() != null && !login.getRtn().equals(0)) {
            throw new BaseException(login.getMessage());
        }

        return this.repository(login.getSession_id(), timeout);
    }

    /**
     * 人脸库检索
     *
     * @param photo base64图片
     */
    public RetrievaResultVO retrieval(String photo) {
        return this.retrieval(photo, WebConstant.TIMEOUT * 1000);
    }


    /**
     * 人脸库检索
     *
     * @param photo   base64图片
     * @param timeout 毫秒
     */
    public RetrievaResultVO retrieval(String photo, Integer timeout) {
        RetrievaResultVO login = this.login(timeout);
        log.info("依图 login: {}", login.toString());
        if (login.getRtn() != null && !login.getRtn().equals(0)) {
            throw new BaseException(login.getMessage());
        }

        RetrievaResultVO repository = this.repository(login.getSession_id(), timeout);
        if (repository != null && !repository.getRtn().equals(0)) {
            throw new BaseException(repository.getMessage());
        }

        List<RepositoryVO> voList = Convert.toList(RepositoryVO.class, repository.getResults());
        RetrievalRepositoryParam param = new RetrievalRepositoryParam() //.setExtra_fields(Lists.newArrayList("custom_field_1"))
                .setOrder(new OrderDTO().setSimilarity(-1))
                .setStart(0).setLimit(1)
                //.setRetrieval_query_id("1")
                .setRetrieval(new RetrievalDTO().setPicture_image_content_base64(photo).setFast(true).setThreshold(threshold)
                        .setRepository_ids(voList.stream().map(RepositoryVO::getId).collect(Collectors.toList())));
        log.info("依图调用参数: {}", JSON.toJSONString(param));
        String body = HttpRequest.post(retrievalUrl).header(HEADERS_KEY, login.getSession_id()).body(JSON.toJSONString(param)).timeout(timeout).execute().body();
        log.info("依图返回结果: {}", body);
        return JSON.parseObject(body, RetrievaResultVO.class);
    }


}
